import requests

# return types
json = 0
str = 1

def request(apiUrl: str, _response: int = 0, headers: set=None) -> set:
    """
    request with no python errors
    """
    try:
        response = requests.get(apiUrl, headers=headers)

        if response.status_code == 200:
            repo_data = response.content

            if _response == 0:
                repo_data = response.json()

            return repo_data
        else:
            print(f"Error: {response.status_code} | {response.text}")

    except Exception as e:
        print(f"Error: {str(e)}")


from gitor import api, _set, gitrequest

def get_repository(self, name: str, raw: bool = False) -> set:
    """
    get a repository
    """
    # if raw wanted then return the raw str
    if raw:
        JSON = gitrequest.request(api.get_repo_url(self.REPO_OWNER, name), gitrequest.json, api.get_headers(self.TOKEN))
        return gitrequest.request(JSON[api.raw], gitrequest.str, api.get_headers(self.TOKEN))
    
    return gitrequest.request(api.get_repo_url(self.REPO_OWNER, name), gitrequest.json, api.get_headers(self.TOKEN))

#def getSubRepositoryByJSON(JSON: set, Name: str, Directory: str):

def get_sub_repository(self, name: str, directory: str, raw: bool = False) -> set:
    DirectoryList = directory.split("/")
    LastRepo = get_repository(self, name)

    if not LastRepo: return
    
    # iterate through all the direcotries
    for i in DirectoryList:
        repo = _set.find_value_in_json(LastRepo, api.name, i)
        if not repo: return
        
        # check if another url is needed
        LastRepo = gitrequest.request(repo["url"], gitrequest.json, api.get_headers(self.TOKEN))
        if not LastRepo: print (f"Error: could not get repo {repo}"); return

    # if raw wanted then return the raw str
    if raw:
        JSON = gitrequest.request(api.get_repo_url(self.REPO_OWNER, name), gitrequest.json, api.get_headers(self.TOKEN))
        return gitrequest.request(LastRepo[api.raw], gitrequest.str, api.get_headers(self.TOKEN))
    
    return LastRepo


name = "name"
raw = "download_url"

def get_repo_url(owner: str, name: str):
    """
    Get the repository url
    """
    owner = owner.lower()
    name = name.lower()
    return f"https://api.github.com/repos/{owner}/{name}/contents"

def get_headers(token: str):
    """
    get headers for private repositories
    """
    return {
        "Authorization": f"token {token}",
        "Accept": "application/vnd.github.v3+json"
    }